@@ -1,5 +1,6 @@
 __all__ = ["check_version", "get_version", "Style", "Vcs", "Version"]
 
+import datetime as dt
 import pkg_resources
 import re
 import shlex
@@ -220,7 +221,12 @@ class Version:
         )
 
     def serialize(
-        self, metadata: bool = None, dirty: bool = False, format: str = None, style: Style = None
+        self,
+        metadata: bool = None,
+        dirty: bool = False,
+        format: str = None,
+        style: Style = None,
+        bump: bool = False,
     ) -> str:
         """
         Create a string from the version info.
@@ -243,12 +249,26 @@ class Version:
             set and no custom format given. If you specify both a style and a
             custom format, then the format will be validated against the
             style's rules.
+        :param bump: If true, increment the last part of the `base` by 1,
+            unless `stage` is set, in which case either increment `revision`
+            by 1 or set it to a default of 2 if there was no revision.
         """
+        base = self.base
+        revision = self.revision
+        if bump:
+            if self.stage is None:
+                base = bump_version(self.base)
+            else:
+                if self.revision is None:
+                    revision = 2
+                else:
+                    revision = self.revision + 1
+
         if format is not None:
             out = format.format(
-                base=self.base,
+                base=base,
                 stage=_blank(self.stage, ""),
-                revision=_blank(self.revision, ""),
+                revision=_blank(revision, ""),
                 distance=_blank(self.distance, ""),
                 commit=_blank(self.commit, ""),
                 dirty="dirty" if self.dirty else "clean",
@@ -271,30 +291,30 @@ class Version:
         pre_parts = []
         if self.stage is not None:
             pre_parts.append(self.stage)
-            if self.revision is not None:
-                pre_parts.append(str(self.revision))
+            if revision is not None:
+                pre_parts.append(str(revision))
         if self.distance > 0:
-            pre_parts.append("post")
+            pre_parts.append("pre" if bump else "post")
             pre_parts.append(str(self.distance))
 
         if style == Style.Pep440:
             if self.distance <= 0:
                 out = serialize_pep440(
-                    self.base, stage=self.stage, revision=self.revision, metadata=meta_parts
+                    base, stage=self.stage, revision=revision, metadata=meta_parts
                 )
             else:
                 out = serialize_pep440(
-                    self.base,
+                    base,
                     stage=self.stage,
-                    revision=self.revision,
-                    post=self.distance,
-                    dev=0,
+                    revision=revision,
+                    post=None if bump else self.distance,
+                    dev=self.distance if bump else 0,
                     metadata=meta_parts,
                 )
         elif style == Style.SemVer:
-            out = serialize_semver(self.base, pre=pre_parts, metadata=meta_parts)
+            out = serialize_semver(base, pre=pre_parts, metadata=meta_parts)
         elif style == Style.Pvp:
-            out = serialize_pvp(self.base, metadata=[*pre_parts, *meta_parts])
+            out = serialize_pvp(base, metadata=[*pre_parts, *meta_parts])
 
         check_version(out, style)
         return out
@@ -324,7 +344,15 @@ class Version:
         code, msg = _run_cmd("git describe --always --dirty")
         dirty = msg.endswith("-dirty")
 
-        code, msg = _run_cmd("git tag --merged HEAD --sort -creatordate")
+        if not dirty:
+            code, msg = _run_cmd("git status --porcelain=v1")
+            if msg.strip() != "":
+                dirty = True
+
+        code, msg = _run_cmd(
+            'git for-each-ref "refs/tags/*" --merged HEAD'
+            ' --format "%(refname)@{%(creatordate:iso-strict)@{%(*committerdate:iso-strict)"'
+        )
         if not msg:
             try:
                 code, msg = _run_cmd("git rev-list --count HEAD")
@@ -332,10 +360,23 @@ class Version:
             except Exception:
                 distance = 0
             return cls("0.0.0", distance=distance, commit=commit, dirty=dirty)
-        tags = msg.splitlines()
+        detailed_tags = []
+        for line in msg.strip().splitlines():
+            parts = line.split("@{")
+            detailed_tags.append(
+                (
+                    parts[0].replace("refs/tags/", "", 1),
+                    _parse_git_timestamp_iso_strict(parts[1]),
+                    None if parts[2] == "" else _parse_git_timestamp_iso_strict(parts[2]),
+                )
+            )
+        tags = [
+            t[0]
+            for t in reversed(sorted(detailed_tags, key=lambda x: x[1] if x[2] is None else x[2]))
+        ]
         tag, base, stage, unmatched = _match_version_pattern(pattern, tags, latest_tag)
 
-        code, msg = _run_cmd("git rev-list --count {}..HEAD".format(tag))
+        code, msg = _run_cmd("git rev-list --count refs/tags/{}..HEAD".format(tag))
         distance = int(msg)
 
         version = cls(base, stage=stage, distance=distance, commit=commit, dirty=dirty)
@@ -868,4 +909,10 @@ def bump_version(base: str, index: int = -1) -> str:
     return ".".join(str(x) for x in bases)
 
 
+def _parse_git_timestamp_iso_strict(raw: str) -> dt.datetime:
+    # Remove colon from timezone offset for pre-3.7 Python:
+    compat = re.sub(r"(.*T.*[-+]\d+):(\d+)", r"\1\2", raw)
+    return dt.datetime.strptime(compat, "%Y-%m-%dT%H:%M:%S%z")
+
+
 __version__ = get_version("dunamai").serialize()
